package de.laliluna.howto.components;

import de.laliluna.howto.model.Phone;
import de.laliluna.howto.services.MessageFactory;

import javax.faces.convert.Converter;
import javax.faces.convert.ConverterException;
import javax.faces.convert.FacesConverter;
import javax.faces.context.FacesContext;
import javax.faces.component.UIComponent;

/**
 * Author: Sebastian Hennebrueder, http://www.laliluna.de
 * Date: 19.10.2009
 */
@FacesConverter(//value ="phoneConverter", 
		forClass =  Phone.class)
public class PhoneConverter implements Converter {

	/**
	 * Actually obsolete as we registered the converter in the faces-config by type
	 */
	public static final String CONVERTER_ID = "de.laliluna.Phone";

	public Object getAsObject(FacesContext facesContext, UIComponent uiComponent, String rawValue) {
		Phone phone = null;
		if (rawValue != null && rawValue.length()>0) {
			String parts[] = rawValue.split("/");

			if (parts.length != 3)
				throw new ConverterException(MessageFactory.getMessage(facesContext,
						"phoneconverter-bad-string", new Phone("49", "123", "456789")));
			phone = new Phone(parts[0], parts[1], parts[2]);
		}
		return phone;
	}

	public String getAsString(FacesContext facesContext, UIComponent uiComponent, Object value) {
		// We are friendly and return at least an empty String
		String stringValue = "";
		if (value != null) {
			// Default components check if the value is a String and return it as it is
			if (value instanceof String)
				stringValue = (String) value;
			else if (value instanceof Phone) {
				Phone phoneValue = (Phone) value;
				stringValue = String.format("%s/%s/%s", phoneValue.getCountryCode(), phoneValue.getAreaPrefix(),
						phoneValue.getCode());
			} else
				throw new ConverterException(MessageFactory.getMessage(facesContext,"phoneconverter-bad-phone"));
		}
		return stringValue;

	}
}
