package de.laliluna.examples.core;

import sun.util.calendar.Gregorian;

import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.ConcurrentSkipListSet;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.ConcurrentHashMap;
import java.util.*;

/**
 * Mar 10, 2009
 *
 * @author Sebastian Hennebrueder
 */

public class DoubleSubmitValidator {
	private static AtomicInteger counter = new AtomicInteger();

	private static ConcurrentMap<Entry, Integer> submittedFormIds = new ConcurrentHashMap<Entry, Integer>();

	private static Random random = new Random();

	public static class Entry {
		private Date timestamp;
		private Integer submittedFormIds;

		public Entry(Integer submittedFormIds) {
			this.submittedFormIds = submittedFormIds;
			this.timestamp = new Date();
		}

		public Date getTimestamp() {
			return timestamp;
		}

		public Integer getSubmittedFormIds() {
			return submittedFormIds;
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || getClass() != o.getClass()) return false;

			Entry entry = (Entry) o;

			if (submittedFormIds != null ? !submittedFormIds.equals(entry.submittedFormIds) : entry.submittedFormIds != null)
				return false;

			return true;
		}

		@Override
		public int hashCode() {
			return submittedFormIds != null ? submittedFormIds.hashCode() : 0;
		}
	}

	public static int getFormCount() {
		return counter.addAndGet(1);
	}

	public static boolean submitted(Integer formId) {
		Entry entry = new Entry(formId);
		Integer existing = submittedFormIds.putIfAbsent(entry, 0);

		// clean up after about 10 submits
		if (random.nextInt(10) == 5) {
			Calendar calendar = GregorianCalendar.getInstance();
			calendar.add(Calendar.HOUR_OF_DAY, -1);
			Date anHourAgo = calendar.getTime();
			List<Entry> toDelete = new ArrayList<Entry>();
			for (Entry e : submittedFormIds.keySet()) {
				if (anHourAgo.after(e.getTimestamp()))
					toDelete.add(entry);
			}
			for (Entry e : toDelete) {
				submittedFormIds.remove(e);
			}
		}

		return existing != null;
	}

	
}
