package de.laliluna.example.pages.ajax;

import org.apache.tapestry5.corelib.components.Form;
import org.apache.tapestry5.ioc.annotations.Inject;
import org.apache.tapestry5.Block;
import org.apache.tapestry5.ValueEncoder;
import org.apache.tapestry5.ajax.MultiZoneUpdate;
import org.apache.tapestry5.internal.services.StringValueEncoder;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.annotations.Persist;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import de.laliluna.example.model.Address;
import de.laliluna.example.services.AddressService;

import java.util.*;

/**
 * Example to update multiple zones with JavaScript.
 * It uses a conditional select - first continents then countries - and at the same time updates the help window.
 * 
 * User: hennebrueder
 * Date: 04.08.2009
 */
public class MultiZone {

	private final Logger logger = LoggerFactory.getLogger(MultiZone.class);

	@Inject
	private Block helpBlock, continentBlock, countryBlock;

	private List<String> continents;

	@Persist
	private String continent;

	@Persist
	private String country;

	private String helpMessage;

	/**
	 * Map of Continent and corresponding countries
	 */
	private Map<String, List<String>> countries = new HashMap<String, List<String>>();

	public MultiZone() {
		// This is not efficient, be might prefer a static storage somewhere
		continents = Arrays.asList("Europe", "Asia", "Australia", "America", "Antarktika");
		countries.put("America", Arrays.asList("Brasil", "Argentina", "United States"));
		countries.put("Europe", Arrays.asList("The Netherlands", "France", "Spain", "Germany"));
		countries.put("Asia", Arrays.asList("Taiwan", "Tailand", "Korea", "China", "Russia"));
		countries.put("Australia", Arrays.asList("Australia"));
		countries.put("Antarktika", Arrays.asList("Nope"));
	}

	public Date getNow() {
		return new Date();
	}

	Object onActionFromUpdate() {
		helpMessage = "Please select a continent";
		return new MultiZoneUpdate("help", helpBlock).add("form", continentBlock);
	}

	Object onSubmitFromContinentForm() {
		helpMessage = "Please select a country (limited choice)";
		return new MultiZoneUpdate("help", helpBlock).add("form", countryBlock);
	}

	Object onSubmitFromCountryForm() {
		helpMessage = String.format("You have selected (%s) in (%s)", country, continent);
		return new MultiZoneUpdate("help", helpBlock).add("form", "");
	}

	public List<String> getContinents() {
		return continents;
	}

	public List<String> getCountries() {
		return countries.get(continent);
	}

	public String getContinent() {
		return continent;
	}

	public void setContinent(String continent) {
		this.continent = continent;
	}

	public String getCountry() {
		return country;
	}

	public void setCountry(String country) {
		this.country = country;
	}

	public String getHelpMessage() {
		return helpMessage;
	}
}
