package de.laliluna.example.pages.input;

import de.laliluna.example.model.Address;
import de.laliluna.example.services.AddressService;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.annotations.IncludeJavaScriptLibrary;
import org.apache.tapestry5.ioc.annotations.Inject;
import org.apache.tapestry5.ValidationException;

import java.util.List;

/**
 * This example shows a normal form with custom converters ({@link CreateCoolAddress#onToClientFromCountry},
 * {@link CreateCoolAddress#onParseClientFromCountry(String)}), a cross field validation
 * {@link CreateCoolAddress#onValidateForm} and how to overwrite the default editor in the CreateCoolAddress.tml .
 * Alternatively to the <t:parameter name="moved">...</t:parameter> in the template, you might use a plain BeanEditForm tag
 * and let the custom editor block for Date be rendered as well. {@link CustomPropertyEditBlocks} 
 *
 * User: hennebrueder
 * Date: 04.08.2009
 */
public class CreateCoolAddress {
	@Property
	private Address address = new Address();

	public List<Address> getAddresses() {
		return addressService.findAll();
	}

	@Inject
	private AddressService addressService;

	Object onSuccess() {
		addressService.create(address);
		return null;
	}

	/**
	 * Send the client Germany if we have germania in the date
	 * @return
	 */
	String onToClientFromCountry() {
		if ("germania".equalsIgnoreCase(address.getCountry()))
			return "Germany";
		else
			return address.getCountry();
	}

	/**
	 * Transform germania to Germany
	 *
	 * @param value
	 * @return
	 */
	Object onParseClientFromCountry(String value) {
		if ("germania".equalsIgnoreCase(value))
			return "Germany";
		else
			return value;
	}

	void onValidateFromCountry(String value) throws ValidationException {
		if (value.equals("bubble")) throw new ValidationException("Bubble is not a valid country");
	}

	void onValidateForm() throws ValidationException {
		if ("bar".equals(address.getName()) && "Germany".equals(address.getCountry()))
			throw new ValidationException("Bar is not a valid name in Germany");
	}


}