package de.laliluna.example.pages.input;

import org.apache.tapestry5.FieldTranslator;
import org.apache.tapestry5.FieldValidator;
import org.apache.tapestry5.ComponentResources;
import org.apache.tapestry5.services.PropertyEditContext;
import org.apache.tapestry5.annotations.Component;
import org.apache.tapestry5.annotations.Environmental;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.corelib.components.TextField;
import org.apache.tapestry5.corelib.components.DateField;
import org.apache.tapestry5.ioc.annotations.Inject;
import org.apache.tapestry5.ioc.Messages;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * This class contains custom editors for the beanEditForm component. It provides a editor
 * for a Phone field and overwrites the default Date editor to allow internationalized input.
 *
 * Author: Sebastian Hennebrueder, http://www.laliluna.de
 * Date: 06.08.2009
 */
public class CustomPropertyEditBlocks {

	@Property
	@Environmental
	private PropertyEditContext context;

	@Inject
	private ComponentResources resources;

	/* A text field to edit phone numbers */

	@Component(parameters =
			{"value=context.propertyValue", "label=prop:context.label",
					"translate=prop:phoneTranslator", "validate=prop:phoneValidator",
					"clientId=prop:context.propertyId", "annotationProvider=context"})
	private TextField phonefield;

	@Inject
	private Messages messages;

	private final Logger logger = LoggerFactory.getLogger(CustomPropertyEditBlocks.class);


	/**
	 * We overwrite the default Date field to look up the date format in the resource bundle
	 */
	@Component(
			parameters = {"value=context.propertyValue", "label=prop:context.label", "clientId=prop:context.propertyid",
					"validate=prop:dateFieldValidator", "format=prop:dateFieldFormat"})
	private DateField dateField;

	public String getDateFieldFormat() {
		String dateFieldFormater = "MM/dd/yyyy";

		if (messages.contains("format.date.field"))
			dateFieldFormater = messages.get("format.date.field");

		return dateFieldFormater;
	}

	public FieldValidator getDateFieldValidator() {
		return context.getValidator(dateField);
	}

	public FieldValidator getPhoneValidator() {
		FieldValidator fieldValidator = context.getValidator(phonefield);
		return fieldValidator;
	}

	public FieldTranslator getPhoneTranslator() {
		FieldTranslator translator = context.getTranslator(phonefield);
		return translator;
	}


}
