package de.laliluna.example.security;

import org.apache.tapestry5.services.*;

import java.io.IOException;

/**
 * This request filter fetches a user from the HTTP session if present and stores it in a ThreadLocal using the
 * {@link de.laliluna.example.security.SecurityService}
 * After the request was processed it cleans up the ThreadLocal, loggs out the user if required or stores back
 * the user to the HTTP session.
 * 
 * Author: Sebastian Hennebrueder, http://www.laliluna.de
 * Date: 05.08.2009
 */
public class SecurityRequestFilter implements RequestFilter {
	private SecurityService securityService;

	public SecurityRequestFilter(SecurityService securityService) {
		this.securityService = securityService;
	}

	/**
	 * Returns true if the request has been handled, false otherwise.
	 */
	public boolean service(Request request, Response response, RequestHandler handler) throws IOException {
		{
			Session session = request.getSession(false);
			if (session != null) {
				ApplicationUser user = (ApplicationUser) session.getAttribute(ApplicationUser.class.getName());
				if (user != null)
					securityService.storeUser(user);
			}
		}
		try {
			handler.service(request, response);
		} finally {
			ApplicationUser user = securityService.getUser();
			// this line is important, app server do reuse threads and of course thread locales
			securityService.cleanStorage();
			if (user == LoggedOutUser.getInstance()) {
				Session session = request.getSession(false);
				if (session != null) {
					session.setAttribute(ApplicationUser.class.getName(), null);
					session.invalidate();
				}
			} else if (user != null) {
				request.getSession(true).setAttribute(ApplicationUser.class.getName(), user);
			}
		}

		return true;
	}
}
