package de.laliluna.example.services;

import de.laliluna.example.model.Address;
import de.laliluna.example.model.AbstractModel;
import de.laliluna.example.security.Protected;

import java.util.List;
import java.util.ArrayList;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * User: hennebrueder
 * Date: 04.08.2009
 */
public class AbstractServiceImpl<M extends AbstractModel> implements AbstractService<M> {
	// we have to trust that this will only be used as singleton
	private ConcurrentMap<Integer, M> modelMap = new ConcurrentHashMap<Integer, M>();
	private AtomicInteger id = new AtomicInteger();


	public List<M> findAll() {
		List<M> result = new ArrayList<M>();
		for (M model : modelMap.values()) {
			result.add((M) model.clone());
		}
		return result;
	}

	public M findById(Integer id) {
		M model = modelMap.get(id);
		return model == null ? null : (M) model.clone();
	}

	/**
	 * Method is thread but not transaction safe
	 * @param model
	 */
	public void update(M model) {
		M existing = modelMap.get(model.getId());
		if(existing == null)
			throw new RuntimeException(String.format("M not found for id (%d)", model.getId()));

		modelMap.put(model.getId(), model);

	}

	public void delete(M model) {
			modelMap.remove(model.getId());
	}

	public void create(M model) {
		model.setId(id.incrementAndGet());
		modelMap.put(model.getId(), model);
	}
}