package de.laliluna.example.services;

import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.Queue;
import java.util.Calendar;
import java.util.Random;


/**
 * Unique form id generator, holding the ids for 180 minutes before removing them
 * For production usage, you might consider to add a DOS limit in order to prevent out of memory problems
 *
 * Author: Sebastian Hennebrueder, http://www.laliluna.de
 * Date: 10.08.2009
 */
public class FormSubmitIdServiceImpl implements FormSubmitIdService {
	private static Queue<Entry> formIdQueue = new ConcurrentLinkedQueue<Entry>();
	private final static int ttlMinutes = 180;
	private final static Random r = new Random();

	class Entry {
		private long timestamp;
		private int id;

		Entry(long timestamp, int id) {
			this.timestamp = timestamp;
			this.id = id;
		}

		public long getTimestamp() {
			return timestamp;
		}

		public int getId() {
			return id;
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (!(o instanceof Entry)) return false;
			Entry entry = (Entry) o;
			return (id == entry.id) ;
		}

		@Override
		public int hashCode() {
			return id;
		}
	}

	public int generateId() {
		cleanUpQueue();
		int id;
		synchronized (r) {
			id = r.nextInt();
		}
		Entry entry = new Entry(Calendar.getInstance().getTimeInMillis(), id);
		formIdQueue.offer(entry);
		return id;
	}

	public boolean checkAndRemove(int id){
		Entry entry = new Entry(-1, id);
		boolean contains = formIdQueue.remove(entry);
		cleanUpQueue();
		return contains;
	}

	private void cleanUpQueue() {
		Calendar calendar = Calendar.getInstance();
		calendar.add(Calendar.MINUTE, -1 * ttlMinutes);
		long expireLimit = calendar.getTimeInMillis();

		FormSubmitIdServiceImpl.Entry entry = formIdQueue.peek();
		while (entry != null) {
			if (entry.getTimestamp() < expireLimit) {
				formIdQueue.remove(entry);
				entry = formIdQueue.peek();
			} else
				break;
		}
	}

}
